"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const base_1 = require("../base");
const fetchCompleted_1 = require("../utils/data/fetchCompleted");
const searchIssues_1 = require("../utils/jira/searchIssues");
const teamUtils_1 = require("../utils/misc/teamUtils");
const initCalendar_1 = require("../utils/velocity/initCalendar");
const insertClosed_1 = require("../utils/velocity/insertClosed");
const insertWeeklyVelocity_1 = require("../utils/velocity/insertWeeklyVelocity");
const jiraUtils_1 = require("../utils/misc/jiraUtils");
const jiraUtils_2 = require("../utils/misc/jiraUtils");
class Velocity extends base_1.default {
    constructor() {
        super(...arguments);
        /*
          Fetch open issues from Jira
        */
        this.fetchOpenIssues = async (userConfig, teamName) => {
            const teamConfig = userConfig.teams.find(t => t.name === teamName);
            if (teamConfig !== undefined) {
                const openIssues = [];
                cli_ux_1.default.action.start('Fetching open issues for team: ' +
                    teamName +
                    ' using JQL: ' +
                    teamConfig.jqlRemaining +
                    ' ');
                const issuesJira = await searchIssues_1.default(userConfig.jira, teamConfig.jqlRemaining, 'labels,' + userConfig.jira.fields.points);
                for (let issue of issuesJira) {
                    issue = Object.assign(Object.assign({}, issue), { points: jiraUtils_2.returnTicketsPoints(issue, userConfig) });
                    openIssues.push(jiraUtils_1.cleanIssue(issue));
                }
                cli_ux_1.default.action.stop(' done');
                return issuesJira;
            }
            return [];
        };
    }
    async run() {
        const userConfig = this.userConfig;
        for (const team of userConfig.teams) {
            const closedIssues = await fetchCompleted_1.default(userConfig, this.config.configDir + '/cache/', team.name);
            console.log('Fetched ' + closedIssues.length + ' completed issues');
            const emptyCalendar = initCalendar_1.default(team.jqlHistory);
            // Extract an array of assignees from the closed issues
            const assignees = closedIssues.reduce((acc, i) => {
                if (i.fields.assignee !== undefined) {
                    if (i.fields.assignee === null && !acc.includes('EMPTY')) {
                        acc.push('EMPTY');
                    }
                    else if (i.fields.assignee !== null && !acc.includes(i.fields.assignee.name)) {
                        acc.push(i.fields.assignee.name);
                    }
                }
                return acc;
            }, []);
            assignees.push('ALL');
            const assigneesVelocity = assignees.map((a) => {
                const assigneesClosedIssues = closedIssues.filter((i) => {
                    if (a === 'ALL') {
                        return true;
                    }
                    if (a === 'EMPTY' && (i.fields.assignee === undefined || i.fields.assignee === null)) {
                        return true;
                    }
                    if (i.fields.assignee !== undefined && i.fields.assignee !== null && i.fields.assignee.name === a) {
                        return true;
                    }
                    return false;
                });
                const calendarWithClosed = insertClosed_1.default(emptyCalendar, assigneesClosedIssues);
                const weeklyVelocity = insertWeeklyVelocity_1.default(calendarWithClosed);
                const trimmedWeeks = weeklyVelocity.map((w) => {
                    if (w.completion.list !== undefined) {
                        w.completion.list = w.completion.list.map((i) => {
                            return {
                                key: i.key,
                                points: i.points
                            };
                        });
                    }
                    delete w.scopeChangeCompletion;
                    return w;
                });
                return {
                    name: a,
                    // closedIssues: calendarWithClosed,
                    weeks: trimmedWeeks,
                };
            });
            const calendarVelocity = {
                assignees,
                velocity: assigneesVelocity,
                updatedAt: new Date().toJSON(),
                host: userConfig.jira.host,
                jqlCompletion: team.jqlCompletion,
            };
            const cacheDir = this.config.configDir + '/cache/';
            const issueFileStream = fs.createWriteStream(path.join(cacheDir, 'assignees-artifacts-' + teamUtils_1.getTeamId(team.name) + '.json'), { flags: 'w' });
            issueFileStream.write(JSON.stringify(calendarVelocity));
            issueFileStream.end();
        }
    }
}
exports.default = Velocity;
Velocity.description = 'Builds velocity stats by day and week';
Velocity.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }) });
exports.formatDate = (dateString) => {
    const day = new Date(dateString);
    day.setUTCHours(12);
    day.setUTCMinutes(0);
    day.setUTCSeconds(0);
    return day;
};
